#include "../foobar2k_sdk/foobar2000/SDK/foobar2000.h"
#include "../foobar2k_sdk/foobar2000/helpers/helpers.h"

#include "kPilotWindow.h"

#define TUTORIAL "Component Tutorial"
#define STEP "Step 2"

static const char* FooClassName = "FooWindowClass";
static const char* FooWindowName = "FooWindow";

/**************************
Providing information about a component.

We can provide some information about our component that 
users will be able to view under Preferences->Components.
**************************/


DECLARE_COMPONENT_VERSION(
	// component name
	"FookPilotInfo",
	// component version
	"0.2",
	// about text, use \n to separate multiple lines
	// If you don't want any about text, set this parameter to NULL.
	"Only to use with kpilot.dll.")




/**************************
Storing settings.

Here settings refer to any information that needs to be
remembered across sessions, like window positions and such.
foobar2000 provides configuration variables to do this,
which come in two different flavors: private and public.
Private configuration variables are derived from cfg_var in
the pfc library; they can only be accessed in the component
that declares them. Public configuration variables are 
dervided from config_var in the main SDK; they can be accessed
from any component. Public configuration variables can be
used from any thread, since they have builtin synchronization,
whereas access to private configuration variables has to be
synchronized by the component where necessary.

We won't use threads in this component, and we don't want
our settings to be publicly available, so we will use the
cfg_var subclasses from the pfc library.

Note: Variables are internally identified by the string you
pass as the first parameter to the constructor. The second
parameter of the contructor is the default value.
**************************/


// --- Step 2 ---
// We will draw some text on our window. Therefore, we
// need a font. Letting the user choose the font is better
// than choosing one ourselves.



// global variables

// window handle of our window or NULL, if it is hidden
static HWND g_wnd = NULL;

static HWND g_hClient = NULL;

// current track
static metadb_handle* current_track = NULL ;
//UINT nIndexPLOfCurrTrack = -1 ;

// forward declarations for some functions

// shows the window
static void g_show_window();

// hides the window
static void g_hide_window();

// window procdure for our window
static LRESULT WINAPI g_windowproc(HWND wnd,UINT msg,WPARAM wp,LPARAM lp);

HWND GetClient()
{
	g_hClient = FindWindow( WindowClass, WindowName );
	
	if (IsWindow(g_hClient))
		return g_hClient;
	else
		return NULL;
}

void Distribute( kPilot2::sKPilot_base *pdata )
{
	COPYDATASTRUCT cpd;

	cpd.cbData = pdata->s_size;
	cpd.dwData = 0;
	cpd.lpData = pdata;

	if (GetClient())
		SendMessage( GetClient(), WM_COPYDATA, (WPARAM)g_wnd, (LPARAM)&cpd );
}

void Log( const char *format, ... )
{
	char buff[1024];
	va_list ap;
	kPilot2::sKPilot_Log Log;

	va_start( ap, format );
	vsprintf( buff, format, ap );
	va_end( ap );

	sprintf( Log.Log, "%s", buff );
	Log.msgID = KP_LOG;

	Distribute( &Log );
}

KPILOTDATA TimeConvert( double time )
{
	int iPos, iSec, iMin, iHour;
	char *buff = (char*)malloc( 16 );
	KPILOTDATA data;

	iPos = (int)time;

	iSec = iPos % 60;		// Sekundy
	iMin = (iPos / 60)%60;		// Minuty
	iHour = iPos / 3600;	// Godziny

	if ( iHour == 0 )
	{
		if ( iMin == 0 )
		{
			if ( iSec < 10 )
				sprintf( buff, "0:0%i", iSec );
			else
				sprintf( buff, "0:%i", iSec );
		}
		else
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:0%i", iMin, iSec );
			else
				sprintf( buff, "%i:%i", iMin, iSec );
		}
	}
	else
	{
		if ( iMin == 0 )
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:00:0%i", iHour, iSec );
			else
				sprintf( buff, "%i:00:%i", iHour, iSec );
		}
		else if ( iMin < 10 )
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:0%i:0%i", iHour, iMin, iSec );
			else
				sprintf( buff, "%i:0%i:%i", iHour, iMin, iSec );
		}
		else
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:%i:0%i", iHour, iMin, iSec );
			else
				sprintf( buff, "%i:%i:%i", iHour, iMin, iSec );
		}
	}

	memcpy( data.text, buff, strlen(buff)+1 );
	free(buff);

	return data;
}

std::string FileSizeConvert( const char* size )
{
	int s, kilo, mega;
	char buff[32], t[16], v[16];
	t[0] = 0;
	v[0] = 0;

	if (!size)
		return 0;

	s = atoi( size );
	s /= 1024;	// kilobajty

	mega = s/1024;
	kilo = s % 1024;

	itoa( kilo, t, 10 );
	strncpy( v, t, 2 );
	v[2] = '\0';
	sprintf( buff, "%i,%s", mega, v);

	return std::string(buff);
}

void Register()
{
	COPYDATASTRUCT cpd;

	cpd.cbData = sizeof(kPilot2::sKPilot_Register);

	cpd.dwData = 0;
	kPilot2::sKPilot_Register reg( KP_REGISTER, "FooBar2000" );        
	cpd.lpData = &reg;

	if (GetClient())
		SendMessage( GetClient(), WM_COPYDATA, (WPARAM)g_wnd, (LPARAM)&cpd );
}


/**************************
Component initialization and shutdown.

Global (static) variables can be assigned an initial value
when loading the DLL by adding an initializer to their
declarations. However, configuration variables don't "know"
their value at that point of time and services aren't available
yet.

On the other hand, a component may need to perform some kind of
cleanup when the application is shutting down. Doing cleanup
when the DLL is unloaded may be too late, since services are no
longer available. One example would be storing the final value
of some setting (window position!) in a configuration variable.

So a properly behaving should component should fully initialze
itself when all components have been loaded, and should perform
any necessary cleanup, before any component is unloaded. For this
purpose exists the initquit callback.
**************************/


// Our component will show its window at initialzation time, when
// the window is enabled. If the window is shown when the
// application exits, the window will be closed (and thus its
// position will be stored).

int g_ID;

class initquit_tutorial1 : public initquit {
	virtual void on_init() 
	{
        g_show_window();
        
		BOOL ok = SendMessage( GetClient(), KP_INIT, (WPARAM)g_wnd, 0);

		if (!ok)
			Register();

		kPilot2::sKPilot_Ex ex;
		ex.msgID = KP_GETVOLUME;
		int Vol = play_control::get()->get_volume();
			
        double temp = (double)Vol;
		double dVol = temp/(-10000);
		dVol = 1 - dVol;
		dVol = dVol*100;
		ex.msgID = KP_GETVOLUME;
		ex.vol = (int)dVol;

		Distribute( &ex );
	}

	virtual void on_quit() {
		// Hide the window without disabling it.
		if (g_wnd)
			DestroyWindow(g_wnd);

		if (current_track)
            current_track->handle_release();
	}

	virtual void on_system_shutdown() {
		// System shutdown is handled like a normal
		// application exit here.
		on_quit();
	}
};

// We have to use a service factory to make our callback known to
// the system. Service factories have to be statically created.
static initquit_factory< initquit_tutorial1 > foo_initquit;




static LRESULT WINAPI g_windowproc(HWND wnd,UINT msg,WPARAM wp,LPARAM lp) {
	kPilot2::sKPilot_Ex data;

	switch (msg) {

	case WM_DESTROY:
		{
			// Remove our window handle from the global variable.
			g_wnd = NULL;
		}
		break;
	case KP_PLAY:
		{
			play_control::get()->play_start( play_control::TRACK_COMMAND_PLAY );
		}
		break;
	case KP_PAUSE:
		{
			play_control::get()->pause();
		}
		break;
	case KP_STOP:
		{
			play_control::get()->play_stop();
		}
		break;
	case KP_NEXT:
		{
			play_control::get()->play_start( play_control::TRACK_COMMAND_NEXT );
		}
		break;
	case KP_PREV:
		{
			play_control::get()->play_start( play_control::TRACK_COMMAND_PREV );
		}
		break;
	case KP_VOLUP:
		{
			play_control::get()->volume_up();
		}
		break;
	case KP_VOLDOWN:
		{
			play_control::get()->volume_down();
		}
		break;
	case KP_GETVOLUME:
		{
			int Vol = play_control::get()->get_volume();
			
            double temp = (double)Vol;
			double dVol = temp/(-10000);
			dVol = 1 - dVol;
			dVol = dVol*100;
			data.msgID = KP_GETVOLUME;
			data.vol = (int)dVol;

			Distribute( &data );
		}
		break;
	case KP_TRACKTIME:
		{
			double dPos = play_control::get()->playback_get_position();

			sprintf( data.szTime, "%s", TimeConvert( dPos ).text);
			data.time = dPos;
			data.msgID = KP_TRACKTIME;

			Distribute( &data );
		}
		break;
	case KP_TRACKTIMEREMAIN:
		{
			double dDuration = play_control::get()->playback_get_length();
			double dPos = play_control::get()->playback_get_position();

			dDuration -= dPos;

			sprintf( data.szTime, "%s", TimeConvert( dDuration ).text );
			data.time = dDuration;
			data.msgID = KP_TRACKTIMEREMAIN;

			Distribute( &data );
		}
		break;
	case KP_TRACKTIMEPERCENT:
		{
			int t = 0;
			double dDuration = play_control::get()->playback_get_length();
			double dPos = play_control::get()->playback_get_position();

			if (dPos)
			{
				dDuration = (dPos/dDuration)*100;
				t = (int)dDuration;
			}
			else
				dDuration = 0;

			sprintf( data.szTime, "%i", t );
			data.time = dDuration;
			data.msgID = KP_TRACKTIMEPERCENT;

			Distribute( &data );
		}
		break;
	case KP_TRACKTIMEPERCENTREMAIN:
		{
			int t = 0;
			double dDuration = play_control::get()->playback_get_length();
			double dPos = play_control::get()->playback_get_position();

			if (dPos)
			{
				dDuration = ((dDuration-dPos)/dDuration)*100;
				t = (int)dDuration;
			}
			else
				dDuration = 0;

			sprintf( data.szTime, "%i", t );
			data.time = dDuration;
			data.msgID = KP_TRACKTIMEPERCENTREMAIN;

			Distribute( &data );
		}
		break;
	case KP_PLAYER:
		{
			const char *version = core_version_info::get_version_string();
			sprintf( data.szPlayer, "%s", version );

			data.msgID = KP_PLAYER;

			Distribute( &data );
		}
		break;
	case KP_BITRATE:
		{
			string8 str;

			if ( current_track )
			{

				metadb_handle* mytrack = current_track->handle_duplicate() ;

				mytrack->handle_format_title( str, "%__bitrate%", NULL ) ;
				convert_utf8_to_ansi( str, data.szBitrate, 32);

				data.msgID = KP_BITRATE;

				Distribute( &data );

				mytrack->handle_release() ;
			}
		}
		break;
	case KP_FILESIZE:
		{
			string8 str;

			if ( current_track )
			{

				metadb_handle* mytrack = current_track->handle_duplicate() ;

				mytrack->handle_format_title( str, "%_filesize%", NULL ) ;
				convert_utf8_to_ansi( str, data.szFileSize, 32);

				sprintf( data.szFileSize, "%s", FileSizeConvert( data.szFileSize ).c_str() );

				data.msgID = KP_FILESIZE;
				Distribute( &data );

				mytrack->handle_release() ;
			}
		}
		break;
	case WM_COPYDATA:
		{
			memcpy( &data, ((PCOPYDATASTRUCT)lp)->lpData, ((PCOPYDATASTRUCT)lp)->cbData );
			switch (data.msgID)
			{
			case KP_SETVOLUME:
				{
					int vol = ( (100 - (int)data.vol) * (-100));
					play_control::get()->set_volume( vol );
				}
				break;
			}
		}
		break;
	case KP_PLAYERSTATUS:
		{
			kPilot2::sKPilot_Ex data;
			data.msgID = KP_PLAYERSTATUS;

			if ( play_control::get()->is_paused() )
				data.status = PAUSED;
			else if ( play_control::get()->is_playing() )
                data.status = PLAYING;
			else
                data.status = STOPPED;

			Distribute( &data );
		}
		break;
	case KP_GETINFO:
		{
			kPilot2::sKPilot_SongChange SongChange;
			string8 str;

			if ( current_track )
			{

				metadb_handle* mytrack = current_track->handle_duplicate() ;

				mytrack->handle_format_title( str, "%title%", NULL ) ;
				convert_utf8_to_ansi( str, SongChange.title, 4096);
				if( !strcmp( SongChange.title, "?" ) )
					sprintf( SongChange.title, "%s", "" );

				mytrack->handle_format_title( str, "%artist%", NULL ) ;
				convert_utf8_to_ansi( str, SongChange.artist, 4096);
				if( !strcmp( SongChange.artist, "?" ) )
					sprintf( SongChange.artist, "%s", "" );

				mytrack->handle_format_title( str, "%album%", NULL ) ;
				convert_utf8_to_ansi( str, SongChange.album, 4096);
				if( !strcmp( SongChange.album, "?" ) )
					sprintf( SongChange.album, "%s", "" );

				mytrack->handle_format_title( str, "%_filename_ext%", NULL ) ;
				convert_utf8_to_ansi( str, SongChange.filename, 4096);

				double dDuration = play_control::get()->playback_get_length();
				sprintf( SongChange.duration, "%s", TimeConvert( dDuration ).text );

				SongChange.msgID = KP_ONSONG;
				if ( play_control::get()->is_paused() )
					SongChange.status = PAUSED;
				else if ( play_control::get()->is_playing() )
					SongChange.status = PLAYING;
				else
					SongChange.status = STOPPED;

				Distribute( &SongChange );

				mytrack->handle_release() ;
			}
		}
	default:
		return uDefWindowProc(wnd,msg,wp,lp);
	}
	return 0;
}


// --- Step 2 ---
// We want to know when the currently playing song changes, so we can
// update the window.
class play_callback_tutorial1 : public play_callback {
public:
	virtual unsigned get_callback_mask() {
		return MASK_on_playback_new_track | MASK_on_playback_stop | MASK_on_playback_pause;
	}

	virtual void on_playback_new_track(metadb_handle * track) {
		
		//KPILOTDATA data;
		kPilot2::sKPilot_SongChange data;
		kPilot2::sKPilot_Ex ex;
		string8 str;

		if ( current_track != NULL )
		{
			current_track->handle_release() ;
		}

		current_track = track->handle_duplicate() ;

		metadb_handle* mytrack = current_track->handle_duplicate() ;

		mytrack->handle_format_title( str, "%title%", NULL ) ;
		convert_utf8_to_ansi( str, data.title, 4096);
		if( !strcmp( data.title, "?" ) )
			sprintf( data.title, "%s", "" );

		mytrack->handle_format_title( str, "%artist%", NULL ) ;
		convert_utf8_to_ansi( str, data.artist, 4096);
		if( !strcmp( data.artist, "?" ) )
			sprintf( data.artist, "%s", "" );

		mytrack->handle_format_title( str, "%album%", NULL ) ;
		convert_utf8_to_ansi( str, data.album, 4096);
		if( !strcmp( data.album, "?" ) )
			sprintf( data.album, "%s", "" );

		mytrack->handle_format_title( str, "%_filename_ext%", NULL ) ;
		convert_utf8_to_ansi( str, data.filename, 4096);

		double dDuration = play_control::get()->playback_get_length();
		sprintf( data.duration, "%s", TimeConvert( dDuration ).text );

		data.msgID = KP_ONSONG;
		data.status = PLAYING;

		Distribute( &data );

		mytrack->handle_release() ;
	}
	virtual void on_playback_stop(play_control::stop_reason reason) {
			
		kPilot2::sKPilot_Ex data;
		
		if ( reason != play_control::STOP_REASON_STARTING_ANOTHER )
		{
            data.status = STOPPED;
            data.msgID = KP_ONSTATECHANGE;
            
			Distribute( &data );
		}
	}
	virtual void on_playback_starting() {}
	virtual void on_playback_seek(double time) {}
	virtual void on_playback_pause(int state) {
		kPilot2::sKPilot_Ex data;

		if (state)
			data.status = PAUSED;
		else
			data.status = PLAYING;

		data.msgID = KP_ONSTATECHANGE;

		Distribute( &data );
	}
	virtual void on_playback_edited(metadb_handle * track) {}
	virtual void on_playback_dynamic_info(const file_info * info,bool b_track_change) {}
	virtual void on_playback_time(metadb_handle * track, double val) {}
	virtual void on_volume_change(int new_val) {}
};

// Here we go again: another service factory for another callback.
static play_callback_factory< play_callback_tutorial1 > foo_play_callback;



static void g_show_window() {
	
	uWNDCLASS wc;
	memset(&wc,0,sizeof(wc));
	wc.style = CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS;
	wc.lpfnWndProc = g_windowproc;
	wc.hInstance = core_api::get_my_instance();
	wc.hCursor = uLoadCursor(0, MAKEINTRESOURCE(IDC_ARROW));
	wc.hbrBackground = 0;
	wc.lpszClassName = FooClassName;
	wc.cbWndExtra = 0; // for storing a font handle

	uRegisterClass(&wc);

	if (!g_wnd) {
		g_wnd = uCreateWindowEx(
			WS_EX_TOOLWINDOW,
			FooClassName,
			FooWindowName,
			WS_POPUP,
			CW_USEDEFAULT, CW_USEDEFAULT,
			200, 200,
			core_api::get_main_window(),
			/* no menu */ 0,
			core_api::get_my_instance(),
			/* no parameter */ 0 );
	}
}

static void g_hide_window() {
	// Destroy the window.
	if (g_wnd)
		DestroyWindow(g_wnd);
}
