#include "stdafx.h"
#include "kPilotWindow.h"

HWND kPilotWindow::m_hWinamp = NULL;
HWND kPilotWindow::m_hWnd = NULL;
HWND kPilotWindow::m_hClient = NULL;
HWND kPilotWindow::m_hPlayList = NULL;
WNDPROC kPilotWindow::m_OldWndProc = NULL;
int kPilotWindow::plugID = NULL;

void kPilotWindow::Register()
{
	COPYDATASTRUCT cpd;
	kPilot2::sKPilot_Ex data;

	cpd.cbData = sizeof(kPilot2::sKPilot_Register);

	cpd.dwData = 0;
	CMDVersion( &data );
	kPilot2::sKPilot_Register reg( KP_REGISTER, data.szPlayer );        
	cpd.lpData = &reg;

	if (GetClient())
		SendMessage( GetClient(), WM_COPYDATA, (WPARAM)m_hWnd, (LPARAM)&cpd );
}

int kPilotWindow::Init( HWND hWinamp, HINSTANCE hInstance )
{
	if ( !IsWindow( hWinamp ) )
		return 0;

	m_hWinamp = hWinamp;

    WNDCLASSEX wce;
    ZeroMemory(&wce, sizeof(WNDCLASSEX));
    wce.cbSize = sizeof(wce);
    wce.style = 0;
    wce.lpfnWndProc = kPilotWindow::WindowProc;
    wce.cbClsExtra = 0;
    wce.cbWndExtra = 0;
    wce.hInstance = hInstance;
    wce.hCursor = LoadCursor(NULL, IDC_ARROW);
    wce.lpszMenuName = NULL;
    wce.lpszClassName = WAWindowClass;
    wce.hbrBackground = GetSysColorBrush ( COLOR_BTNFACE );
    wce.hIcon = LoadIcon(0, IDI_APPLICATION);
    wce.hIconSm = LoadIcon(0, IDI_WINLOGO);
    RegisterClassEx(&wce);

	m_hWnd = CreateWindowEx (WS_EX_TOOLWINDOW, WAWindowClass, WAWindowName, WS_POPUP, 0, 0, 10, 10, (HWND) m_hWinamp, (HMENU) NULL, hInstance, NULL);

	HWND m_hPlayList = FindWindow("Winamp PE",NULL);
	m_OldWndProc = (WNDPROC)SetWindowLong( m_hPlayList, GWL_WNDPROC, (LONG)PlayListProc ); 

	BOOL ok = (BOOL)SendMessage( GetClient(), KP_INIT, (WPARAM)m_hWnd, 0 );
	if (!ok)
		Register();

	kPilot2::sKPilot_Ex ex;
	ex.msgID = KP_GETVOLUME;
	ex.vol = CMDGetVolume();

	Distribute( &ex );

	SetTimer( m_hWnd, NULL, 1000, NULL );

	return 0;
}

int kPilotWindow::DeInit()
{
	KillTimer( m_hWnd, 0 );
	DestroyWindow( m_hWnd );
	return 0;
}

LRESULT CALLBACK kPilotWindow::PlayListProc( HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam )
{
	if (iMsg == WM_USER && wParam == 0x29A)
	{
		static LPARAM LastSong(-1);
		if( (LastSong != lParam) && (lParam < 1000000) )
		{
			LastSong = lParam;
			OnSongChange();
		}
	}
    return CallWindowProc( m_OldWndProc, hWnd, iMsg, wParam, lParam); 
}

void kPilotWindow::OnSongChange()
{
	kPilot2::sKPilot_SongChange data;
	data.msgID = KP_ONSONG;

	CMDTrackName( &data );
	CMDArtistName( &data );
	CMDTrackDuration( &data );
	CMDDiscName( &data );
	CMDFileName( &data );
	data.status = PLAYING;

	Distribute( &data );
}

LRESULT CALLBACK kPilotWindow::WindowProc( HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam )
{
	//KPILOTDATA data;
	kPilot2::sKPilot_Ex data;

	switch(iMsg)
	{
	case KP_PLAY:
		CMDPlay();
		break;
	case KP_PAUSE:
		CMDPause();
		break;
	case KP_STOP:
		CMDStop();
		break;
	case KP_NEXT:
		CMDNext();
		break;
	case KP_PREV:
		CMDPrev();
		break;
	case KP_VOLUP:
		CMDVolUp();
		break;
	case KP_VOLDOWN:
		CMDVolDown();
		break;
	case KP_TRACKTIME:
		{
			CMDTrackTime( &data );
			data.msgID = KP_TRACKTIME;

			Distribute( &data );
		}
		break;
	case KP_TRACKTIMEREMAIN:
		{
			CMDTrackTimeRemain( &data );
			data.msgID = KP_TRACKTIMEREMAIN;

			Distribute( &data );
		}
		break;
	case KP_TRACKTIMEPERCENT:
		{
			CMDTrackTimePercent( &data );
			data.msgID = KP_TRACKTIMEPERCENT;

			Distribute( &data );
		}
		break;
	case KP_TRACKTIMEPERCENTREMAIN:
		{
			CMDTrackTimePercentRemain( &data );
			data.msgID = KP_TRACKTIMEPERCENTREMAIN;

			Distribute( &data );
		}
		break;
	case KP_PLAYERSTATUS:
		{
			CMDPlayerStatus( &data );
			data.msgID = KP_PLAYERSTATUS;

			Distribute( &data );
		}
		break;
	case KP_GETVOLUME:
		{
			data.vol = CMDGetVolume();
			data.msgID = KP_GETVOLUME;

			Distribute( &data );
		}
		break;
	case KP_PLAYER:
		{
			CMDVersion( &data );
			data.msgID = KP_PLAYER;

			Distribute( &data );
		}
		break;
	case KP_GETINFO:
		{
			OnSongChange();
		}
		break;
	case WM_TIMER:
		{
			static int state;
			int status = (int)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_ISPLAYING);

			if (state != status)
			{
				state = status;
				CMDPlayerStatus( &data );
				data.msgID = KP_ONSTATECHANGE;

				Distribute( &data );
			}
		}
		break;
	case KP_BITRATE:
		{
			CMDBitRate( &data );
			data.msgID = KP_BITRATE;

			Distribute( &data );
			break;
		}
	case KP_FILESIZE:
		{
			CMDFileSize( &data );
			data.msgID = KP_FILESIZE;

			Distribute( &data );
			break;
		}
	case WM_COPYDATA:
		{
			memcpy( &data, ((PCOPYDATASTRUCT)lParam)->lpData, ((PCOPYDATASTRUCT)lParam)->cbData );
			switch (data.msgID)
			{
			case KP_SETVOLUME:
				{
					double dVol = data.vol;
					dVol /= 100;
                    dVol *= 255;

					CMDSetVolume( (int)dVol );
				}
				break;
			}
		}
	default:
		return DefWindowProc( hWnd, iMsg, wParam, lParam);
	}
	return 0;
}

void kPilotWindow::CMDPlay()
{
	PostMessage( m_hWinamp, WM_COMMAND, WINAMP_PLAY, 0 );
}

void kPilotWindow::CMDPause()
{
	PostMessage( m_hWinamp, WM_COMMAND, WINAMP_PAUSE, 0 );
}

void kPilotWindow::CMDStop()
{
	PostMessage( m_hWinamp, WM_COMMAND, WINAMP_STOP, 0 );
}

void kPilotWindow::CMDNext()
{
	PostMessage( m_hWinamp, WM_COMMAND, WINAMP_FASTFWD, 0 );
}

void kPilotWindow::CMDPrev()
{
	PostMessage( m_hWinamp, WM_COMMAND, WINAMP_REWIND, 0 );
}

void kPilotWindow::CMDVolUp()
{
	PostMessage( m_hWinamp, WM_COMMAND, WINAMP_VOLUMEUP, 0);
}

void kPilotWindow::CMDVolDown()
{
	PostMessage( m_hWinamp, WM_COMMAND, WINAMP_VOLUMEDOWN, 0);
}

void kPilotWindow::CMDSetVolume(int Val)
{
	if(Val < 0)
	{
		Val = 0;
	}
	if(Val > 255)
	{
		Val = 255;
	}
	PostMessage( m_hWinamp, WM_WA_IPC, (WPARAM)(long)Val, IPC_SETVOLUME);
}

void kPilotWindow::CMDTrackDuration(kPilot2::sKPilot_Ex * pData)
{
	long Val;
	kPilot2::sKPilot_Ex data;

	*&data = *pData;

	*&Val = (long)SendMessage( m_hWinamp, WM_WA_IPC, 1, IPC_GETOUTPUTTIME );

	sprintf( data.szTime, "%s", TimeConvert(Val).text );
	data.time = (double)Val;
	*pData = *&data;
}

void kPilotWindow::CMDTrackDuration(kPilot2::sKPilot_SongChange * pData)
{
	long Val;
	kPilot2::sKPilot_SongChange data;

	*&data = *pData;

	*&Val = (long)SendMessage( m_hWinamp, WM_WA_IPC, 1, IPC_GETOUTPUTTIME );

	if (Val < 0 )
		return;

	sprintf( data.duration, "%s", TimeConvert(Val).text );
	*pData = *&data;
}

KPILOTDATA kPilotWindow::TimeConvert( long time )
{
	int iPos, iSec, iMin, iHour;
	char *buff = (char*)malloc( 16 );
	KPILOTDATA data;
	sprintf( data.text, "%s", "" );

	if (time < 0)
		return data;

	iPos = (int)time;

	iSec = iPos % 60;		// Sekundy
	iMin = (iPos / 60)%60;		// Minuty
	iHour = iPos / 3600;	// Godziny

	if ( iHour == 0 )
	{
		if ( iMin == 0 )
		{
			if ( iSec < 10 )
				sprintf( buff, "0:0%i", iSec );
			else
				sprintf( buff, "0:%i", iSec );
		}
		else
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:0%i", iMin, iSec );
			else
				sprintf( buff, "%i:%i", iMin, iSec );
		}
	}
	else
	{
		if ( iMin == 0 )
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:00:0%i", iHour, iSec );
			else
				sprintf( buff, "%i:00:%i", iHour, iSec );
		}
		else if ( iMin < 10 )
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:0%i:0%i", iHour, iMin, iSec );
			else
				sprintf( buff, "%i:0%i:%i", iHour, iMin, iSec );
		}
		else
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:%i:0%i", iHour, iMin, iSec );
			else
				sprintf( buff, "%i:%i:%i", iHour, iMin, iSec );
		}
	}

	memcpy( data.text, buff, strlen(buff)+1 );
	free(buff);

	return data;
}

HWND kPilotWindow::GetClient()
{
	m_hClient = FindWindow( WindowClass, WindowName );
	
	if (IsWindow(m_hClient))
		return m_hClient;
	else
		return NULL;
}

void kPilotWindow::Distribute( kPilot2::sKPilot_base * pdata )
{
	COPYDATASTRUCT cpd;

	cpd.cbData = pdata->s_size;
	cpd.dwData = 0;
	cpd.lpData = pdata;

	if (GetClient())
		SendMessage( GetClient(), WM_COPYDATA, (WPARAM)m_hWnd, (LPARAM)&cpd );
}

void kPilotWindow::CMDTrackTime( kPilot2::sKPilot_Ex * pData )
{
	long Val;
	kPilot2::sKPilot_Ex data;
	*&data = *pData;

	*&Val = (long)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_GETOUTPUTTIME );

	Val /= 1000;
	sprintf( data.szTime, "%s", TimeConvert(Val).text );
	data.time = (double)Val;

	*pData = *&data;
}

void kPilotWindow::CMDTrackTimeRemain( kPilot2::sKPilot_Ex * pData )
{
	long Val1;
	long Val2;
	kPilot2::sKPilot_Ex data;

	*&Val1 = (long)SendMessage( m_hWinamp, WM_WA_IPC, 1, IPC_GETOUTPUTTIME );
	*&Val2 = (long)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_GETOUTPUTTIME );

	/* Co qoofa za bzdurne wykonanie w winampie,
	Val1 to czas cakowity utowru w sekundach
	Val2 to aktualna pozycja w milisekundach
	*/
	Val2 /= 1000;
	Val1 -= Val2;
	sprintf( data.szTime, "%s", TimeConvert(Val1).text );
	data.time = (double)Val1;

	*pData = *&data;
}

void kPilotWindow::CMDTrackTimePercent( kPilot2::sKPilot_Ex * pData )
{
	long Val1;
	long Val2;
	double d;
	int t;
	kPilot2::sKPilot_Ex data;

	*&Val1 = (long)SendMessage( m_hWinamp, WM_WA_IPC, 1, IPC_GETOUTPUTTIME );
	*&Val2 = (long)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_GETOUTPUTTIME );

	/* Co qoofa za bzdurne wykonanie w winampie,
	Val1 to czas cakowity utowru w sekundach
	Val2 to aktualna pozycja w milisekundach
	*/
	if (Val2)
	{
		Val2 /= 1000;
		d = (double)Val2;
		d /= Val1;
		d *= 100;
		t = (int)d;
	}
	else
	{
		t = 0;
		Val1 = 0;
	}

	sprintf( data.szTime, "%i", t );
	data.time = Val1;

	*pData = *&data;
}

void kPilotWindow::CMDTrackTimePercentRemain( kPilot2::sKPilot_Ex * pData )
{
	long Val1;
	long Val2;
	double d;
	int t;
	kPilot2::sKPilot_Ex data;

	*&Val1 = (long)SendMessage( m_hWinamp, WM_WA_IPC, 1, IPC_GETOUTPUTTIME );
	*&Val2 = (long)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_GETOUTPUTTIME );

	/* Co qoofa za bzdurne wykonanie w winampie,
	Val1 to czas cakowity utowru w sekundach
	Val2 to aktualna pozycja w milisekundach
	*/
	if (Val2)
	{
		Val2 /= 1000;
		Val2 = Val1-Val2;
		d = (double)Val2;
		d /= Val1;
		d *= 100;
		t = (int)d;
	}
	else
	{
		t = 0;
		Val1 = 0;
	}

	sprintf( data.szTime, "%i", t );
	data.time = Val1;

	*pData = *&data;
}

void kPilotWindow::CMDVersion( kPilot2::sKPilot_Ex * pData )
{
	long Val;
	int i;
	kPilot2::sKPilot_Ex data;

	*&Val = (long)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_GETVERSION );

	i = (int) Val;
	sprintf( data.szPlayer, "WinAmp %i", i );

	*pData = *&data;
}

void kPilotWindow::CMDTrackName( kPilot2::sKPilot_SongChange * pData )
{
	long PlayListPos;
	kPilot2::sKPilot_SongChange data;
	char buff[1024];
	extendedFileInfoStruct einfo;

	*&data = *pData;

	*&PlayListPos = (long)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_GETLISTPOS);
	einfo.filename = (char *)SendMessage( m_hWinamp, WM_WA_IPC, PlayListPos, IPC_GETPLAYLISTFILE);
	einfo.metadata = "Title";
	einfo.ret = buff;
	einfo.retlen = 1024;

	SendMessage( m_hWinamp, WM_WA_IPC, (WPARAM)&einfo, IPC_GET_EXTENDED_FILE_INFO);
        
	memcpy( data.title, buff, strlen(buff)+1);

	*pData = *&data;
}

void kPilotWindow::CMDArtistName( kPilot2::sKPilot_SongChange * pData )
{
	long PlayListPos;
	kPilot2::sKPilot_SongChange data;
	char buff[1024];
	extendedFileInfoStruct einfo;

	*&data = *pData;

	*&PlayListPos = (long)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_GETLISTPOS);
	einfo.filename = (char *)SendMessage( m_hWinamp, WM_WA_IPC, PlayListPos, IPC_GETPLAYLISTFILE);;
	einfo.metadata = "Artist";
	einfo.ret = buff;
	einfo.retlen = 1024;

	SendMessage( m_hWinamp, WM_WA_IPC, (WPARAM)&einfo, IPC_GET_EXTENDED_FILE_INFO);

	memcpy( data.artist, buff, strlen(buff)+1);

	*pData = *&data;
}

void kPilotWindow::CMDDiscName( kPilot2::sKPilot_SongChange * pData )
{
	long PlayListPos;
	kPilot2::sKPilot_SongChange data;
	char buff[1024];
	extendedFileInfoStruct einfo;

	*&data = *pData;

	*&PlayListPos = (long)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_GETLISTPOS);
	einfo.filename = (char *)SendMessage( m_hWinamp, WM_WA_IPC, PlayListPos, IPC_GETPLAYLISTFILE);;
	einfo.metadata = "Album";
	einfo.ret = buff;
	einfo.retlen = 1024;

	SendMessage( m_hWinamp, WM_WA_IPC, (WPARAM)&einfo, IPC_GET_EXTENDED_FILE_INFO);

	memcpy( data.album, buff, strlen(buff)+1);

	*pData = *&data;
}

void kPilotWindow::CMDFileName( kPilot2::sKPilot_SongChange * pData )
{
	long PlayListPos;
	kPilot2::sKPilot_SongChange data;
	char *ptr;
	PSTR pszMem;
	int cTxtLen;
	std::string temp;

	*&data = *pData;

	*&PlayListPos = (long)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_GETLISTPOS);
	ptr = (char *)SendMessage( m_hWinamp, WM_WA_IPC, PlayListPos, IPC_GETPLAYLISTFILE);

	if (!ptr)
	{
		cTxtLen = GetWindowTextLength( m_hWinamp ); 

		pszMem = (PSTR) VirtualAlloc((LPVOID) NULL, 
			(DWORD) (cTxtLen + 1), MEM_COMMIT, 
			PAGE_READWRITE); 
		GetWindowText( m_hWinamp, pszMem, cTxtLen + 1);

		temp = pszMem;
		int pos = (int)temp.find("Winamp");
		temp = temp.substr(0, pos-3);

		sprintf( data.filename, "%s", temp.c_str() );

		*pData = *&data;

		VirtualFree(pszMem, 0, MEM_RELEASE); 
	}

	if (!strstr( ptr, "\\" ))
	{
		memcpy( data.filename, "\0", 1);
		return;
	}

	// tylko nazwa pliku z extension
	ptr = strrchr( ptr, '\\' );
	// dodatkowo trzeba usun '\'
	ptr++;

	memcpy( data.filename, ptr, strlen(ptr)+1);

	*pData = *&data;
}

void kPilotWindow::CMDPlayerStatus( kPilot2::sKPilot_Ex * pData )
{
	kPilot2::sKPilot_Ex data;
	int status;
	status = (int)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_ISPLAYING);

	switch (status)
	{
	case 0:
		data.status = (int)STOPPED;
		break;
	case 1:
		data.status = (int)PLAYING;
		break;
	case 3:
		data.status = (int)PAUSED;
		break;
	}

	*pData = *&data;
}

int kPilotWindow::CMDGetVolume( )
{
	long Val;

	*&Val = (long)SendMessage( m_hWinamp, WM_WA_IPC, (-666), IPC_SETVOLUME);

	double dVol = (double)Val;
	dVol = dVol/255;
	dVol *= 100;

	return (int)dVol;
}

void kPilotWindow::CMDBitRate( kPilot2::sKPilot_Ex * pData )
{
	int bitrate;
	kPilot2::sKPilot_Ex data;
	
	bitrate = (int)SendMessage( m_hWinamp, WM_WA_IPC, 1, IPC_GETINFO );
	
	sprintf( data.szBitrate, "%i", bitrate );

	*pData = *&data;
}

std::string kPilotWindow::FileSizeConvert( int size )
{
	int s, kilo, mega;
	char buff[16], t[4], v[4];
	t[0] = 0;
	v[0] = 0;

	if (!size)
		return 0;

	s = size;
	s /= 1024;	// kilobajty

	mega = s/1024;
	kilo = s % 1024;

	itoa( kilo, t, 10 );
	strncpy( v, t, 2 );
	v[2] = '\0';
	sprintf( buff, "%i,%s", mega, v);

	return std::string(buff);
}

void kPilotWindow::CMDFileSize( kPilot2::sKPilot_Ex *pData )
{
	long PlayListPos;
	kPilot2::sKPilot_Ex data;
	char *ptr;
	HANDLE hFile;
	DWORD dwSize;

	*&data = *pData;

	*&PlayListPos = (long)SendMessage( m_hWinamp, WM_WA_IPC, 0, IPC_GETLISTPOS);
	ptr = (char *)SendMessage( m_hWinamp, WM_WA_IPC, PlayListPos, IPC_GETPLAYLISTFILE); 

	hFile = CreateFile((LPCTSTR)ptr,           // open MYFILE.TXT 
                GENERIC_READ,              // open for reading 
                FILE_SHARE_READ,           // share for reading 
                NULL,                      // no security 
                OPEN_EXISTING,             // existing file only 
                FILE_ATTRIBUTE_NORMAL,     // normal file 
                NULL);                     // no attr. template 
	
	dwSize = GetFileSize(hFile, NULL);
	CloseHandle( hFile );

	sprintf( data.szFileSize, "%s", FileSizeConvert( (int)dwSize ).c_str() );

	*pData = *&data;
}