/////////////////////////////////////////////////////////////////////////////
//
// WMPkPilotInfo.cpp : Implementation of CWMPkPilotInfo
// Copyright (c) Microsoft Corporation. All rights reserved.
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "WMPkPilotInfo.h"

/////////////////////////////////////////////////////////////////////////////
// CWMPkPilotInfo::CWMPkPilotInfo
// Constructor

CWMPkPilotInfo::CWMPkPilotInfo()
{
    m_dwAdviseCookie = 0;
}

/////////////////////////////////////////////////////////////////////////////
// CWMPkPilotInfo::~CWMPkPilotInfo
// Destructor

CWMPkPilotInfo::~CWMPkPilotInfo()
{
	kPilotInfo::Stop();
}

/////////////////////////////////////////////////////////////////////////////
// CWMPkPilotInfo:::FinalConstruct
// Called when an plugin is first loaded. Use this function to do one-time
// intializations that could fail instead of doing this in the constructor,
// which cannot return an error.

HRESULT CWMPkPilotInfo::FinalConstruct()
{
    return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
// CWMPkPilotInfo:::FinalRelease
// Called when a plugin is unloaded. Use this function to free any
// resources allocated in FinalConstruct.

void CWMPkPilotInfo::FinalRelease()
{
    ReleaseCore();
}

/////////////////////////////////////////////////////////////////////////////
// CWMPkPilotInfo::SetCore
// Set WMP core interface

HRESULT CWMPkPilotInfo::SetCore(IWMPCore *pCore)
{
    HRESULT hr = S_OK;

    // release any existing WMP core interfaces
    ReleaseCore();

    // If we get passed a NULL core, this  means
    // that the plugin is being shutdown.

    if (pCore == NULL)
    {
        return S_OK;
    }

    m_spCore = pCore;

	kPilotInfo::Start(_Module.GetModuleInstance(), pCore);

    // connect up the event interface
    CComPtr<IConnectionPointContainer>  spConnectionContainer;

    hr = m_spCore->QueryInterface( &spConnectionContainer );

    if (SUCCEEDED(hr))
    {
        hr = spConnectionContainer->FindConnectionPoint( __uuidof(IWMPEvents), &m_spConnectionPoint );
    }

    if (SUCCEEDED(hr))
    {
        hr = m_spConnectionPoint->Advise( GetUnknown(), &m_dwAdviseCookie );

        if ((FAILED(hr)) || (0 == m_dwAdviseCookie))
        {
            m_spConnectionPoint = NULL;
        }
    }

    return hr;
}

/////////////////////////////////////////////////////////////////////////////
// CWMPkPilotInfo::ReleaseCore
// Release WMP core interfaces

void CWMPkPilotInfo::ReleaseCore()
{
    if (m_spConnectionPoint)
    {
        if (0 != m_dwAdviseCookie)
        {
            m_spConnectionPoint->Unadvise(m_dwAdviseCookie);
            m_dwAdviseCookie = 0;
        }
        m_spConnectionPoint = NULL;
    }

    if (m_spCore)
    {
        m_spCore = NULL;
    }
}


/////////////////////////////////////////////////////////////////////////////
// CWMPkPilotInfo::GetProperty
// Get plugin property

HRESULT CWMPkPilotInfo::GetProperty(const WCHAR *pwszName, VARIANT *pvarProperty)
{
    if (NULL == pvarProperty)
    {
        return E_POINTER;
    }

    return E_NOTIMPL;
}

/////////////////////////////////////////////////////////////////////////////
// CWMPkPilotInfo::SetProperty
// Set plugin property

HRESULT CWMPkPilotInfo::SetProperty(const WCHAR *pwszName, const VARIANT *pvarProperty)
{
    return E_NOTIMPL;
}
