#include "stdafx.h"
#include "kPilotInfo.h"

HWND kPilotInfo::m_hWnd = NULL;
CComPtr<IWMPCore> kPilotInfo::m_pCore = NULL;
UINT_PTR kPilotInfo::m_idTimer = NULL;
HWND kPilotInfo::m_hPilot = NULL;
int kPilotInfo::plugID = NULL;

void kPilotInfo::Register()
{
	COPYDATASTRUCT cpd;

	cpd.cbData = sizeof(kPilot2::sKPilot_Register);

	cpd.dwData = 0;
	kPilot2::sKPilot_Register reg( KP_REGISTER, "Windows Media Player" );        
	cpd.lpData = &reg;

	if (GetClient())
		SendMessage( GetClient(), WM_COPYDATA, (WPARAM)m_hWnd, (LPARAM)&cpd );
}
BOOL kPilotInfo::Start(HINSTANCE hInstance, IWMPCore *pCore)
{
	m_pCore = pCore;

	WNDCLASSEX wce;
    ZeroMemory(&wce, sizeof(WNDCLASSEX));
    wce.cbSize = sizeof(wce);
    wce.style = 0;
	wce.lpfnWndProc = kPilotInfo::WindowProc;
    wce.cbClsExtra = 0;
    wce.cbWndExtra = 0;
    wce.hInstance = hInstance;
    wce.hCursor = LoadCursor(NULL, IDC_ARROW);
    wce.lpszMenuName = NULL;
    wce.lpszClassName = WMP9Class;
    wce.hbrBackground = GetSysColorBrush ( COLOR_BTNFACE );
    wce.hIcon = LoadIcon(0, IDI_APPLICATION);
    wce.hIconSm = LoadIcon(0, IDI_WINLOGO);

    if (!RegisterClassEx(&wce))
		return FALSE;
	
	m_hWnd = CreateWindowEx ( WS_EX_TOOLWINDOW, WMP9Class, WMP9Name, WS_POPUP, 0, 0, 10, 10, 0, (HMENU) NULL, hInstance, NULL);

	BOOL ok = (BOOL)SendMessage( GetClient(), KP_INIT, (WPARAM)m_hWnd, 0 );

	if (!ok)
		Register();

	kPilot2::sKPilot_Ex ex;
	ex.msgID = KP_GETVOLUME;
	
	long lVolume;
	CMDGetVolume( &lVolume );
	ex.vol = (int)lVolume;

	DistributeData( &ex );
	return TRUE;
}

void kPilotInfo::Stop()
{
	DestroyWindow( m_hWnd );

	if (m_pCore)
		m_pCore = NULL;
}

LRESULT kPilotInfo::WindowProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	kPilot2::sKPilot_Ex data;

	switch (uMsg)
	{
	case KP_PLAY:
		return CMDPlay();
	case KP_PAUSE:
		return CMDPause();
	case KP_STOP:
		return CMDStop();
	case KP_NEXT:
		return CMDNext();
	case KP_PREV:
		return CMDPrev();
	case KP_VOLUP:
		return CMDVolumeUp();
	case KP_VOLDOWN:
		return CMDVolumeDown();
	case KP_GETVOLUME:
		{
			long plVolume;
			CMDGetVolume( &plVolume );

			data.msgID = KP_GETVOLUME;
			data.vol = (int)plVolume;

			DistributeData ( &data );
			break;
		}
	case KP_TRACKTIME:
		{
			CMDTrackTime( &data );
			data.msgID = KP_TRACKTIME;

			DistributeData ( &data );
			break;
		}
	case KP_TRACKTIMEREMAIN:
		{
			CMDTrackTimeRemain( &data );
			data.msgID = KP_TRACKTIMEREMAIN;

			DistributeData( &data );
			break;
		}
	case KP_TRACKTIMEPERCENT:
		{
			CMDTrackTimePercent( &data );
			data.msgID = KP_TRACKTIMEPERCENT;

			DistributeData( &data );
			break;
		}
	case KP_TRACKTIMEPERCENTREMAIN:
		{
			CMDTrackTimePercentRemain( &data );
			data.msgID = KP_TRACKTIMEPERCENTREMAIN;

			DistributeData( &data );
			break;
		}
	case KP_PLAYERSTATUS:
		{
			CMDPlayerStatus( &data );
			data.msgID = KP_PLAYERSTATUS;

			DistributeData( &data );
			break;
		}
	case KP_GETINFO:
		{
			OnSongChange();
		}
		break;
	case KP_PLAYER:
		{
			CMDPlayerVersion( &data );
			data.msgID = KP_PLAYER;

			DistributeData( &data );
			break;
		}
	case KP_BITRATE:
		{
			CMDBitRate( &data );
			data.msgID = KP_BITRATE;

			DistributeData( &data );
			break;
		}
	case KP_FILESIZE:
		{
			CMDFileSize( &data );
			data.msgID = KP_FILESIZE;

			DistributeData( &data );
			break;
		}
	case WM_COPYDATA:
		{
			memcpy( &data, ((PCOPYDATASTRUCT)lParam)->lpData, ((PCOPYDATASTRUCT)lParam)->cbData );
			switch (data.msgID)
			{
			case KP_SETVOLUME:
				{
					CMDSetVolume( (long)data.vol );
				}
				break;
			}
			break;
		}
	default:
		return DefWindowProc( hWnd, uMsg, wParam, lParam );
	}

	return 0;
}

void kPilotInfo::OnStateChange( PlayerState state )
{
	kPilot2::sKPilot_Ex ex;
	Log( "OnStateChange! state = %i", state );

	ex.msgID = KP_ONSTATECHANGE;
	ex.status = (int)state;

	DistributeData( &ex );
}

void kPilotInfo::OnSongChange()
{
	CComPtr<IWMPMedia> pMedia;
	HRESULT hr = S_OK;
	BSTR bstrVal;
	char temp[4096];
	std::string buff;
	kPilot2::sKPilot_SongChange data;

	if (SUCCEEDED( hr ) )
	{
		hr = m_pCore->get_currentMedia( &pMedia );
	}

	if (SUCCEEDED( hr ) )
	{
		if (pMedia)
            hr = pMedia->get_durationString( &bstrVal );
		else
			hr = S_FALSE;
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)data.duration, SysStringLen(bstrVal)+1, NULL, NULL );
	}

	if ( SUCCEEDED(hr) )
	{
		hr = pMedia->getItemInfo( L"Title", &bstrVal );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)data.title, SysStringLen(bstrVal)+1, NULL, NULL );
	}

	if ( SUCCEEDED(hr) )
	{
		hr = pMedia->getItemInfo( L"Author", &bstrVal );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)data.artist, SysStringLen(bstrVal)+1, NULL, NULL );
	}

	if ( SUCCEEDED(hr) )
	{
		hr = pMedia->getItemInfo( L"Album", &bstrVal );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)data.album, SysStringLen(bstrVal)+1, NULL, NULL );
	}

	if ( SUCCEEDED(hr) )
	{
		hr = pMedia->get_sourceURL( &bstrVal );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)temp, SysStringLen(bstrVal)+1, NULL, NULL );
		buff = temp;
		int pos = (int)buff.find_last_of( '\\' );
		buff = buff.substr( pos+1, buff.length()-pos-1 );
		sprintf( data.filename, "%s", buff.c_str() );
	}

	Log( "OnSongChange! album=%s|artist=%s|duration=%s|filename=%s|status=%i|title=%s|", data.album, data.artist, data.duration, data.filename, data.status, data.title );

	if ( SUCCEEDED(hr) )
	{
		data.msgID = KP_ONSONG;
		data.status = PLAYING;
		DistributeData( &data );
	}
}

void kPilotInfo::OnSongChange( IDispatch *Item )
{
	CComPtr<IWMPMedia> pMedia;
	HRESULT hr = S_OK;
	BSTR bstrVal;
	std::string buff;
	char temp[4096];
	kPilot2::sKPilot_SongChange data;

	if (SUCCEEDED( hr ) )
	{
		hr = Item->QueryInterface( &pMedia );
	}

	if (SUCCEEDED( hr ) )
	{
		pMedia->get_durationString( &bstrVal );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)data.duration, SysStringLen(bstrVal)+1, NULL, NULL );
	}

	if ( SUCCEEDED(hr) )
	{
		hr = pMedia->getItemInfo( L"Title", &bstrVal );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)data.title, SysStringLen(bstrVal)+1, NULL, NULL );
	}

	if ( SUCCEEDED(hr) )
	{
		hr = pMedia->getItemInfo( L"Author", &bstrVal );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)data.artist, SysStringLen(bstrVal)+1, NULL, NULL );
	}

	if ( SUCCEEDED(hr) )
	{
		hr = pMedia->getItemInfo( L"Album", &bstrVal );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)data.album, SysStringLen(bstrVal)+1, NULL, NULL );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)temp, SysStringLen(bstrVal)+1, NULL, NULL );
		buff = temp;
		int pos = (int)buff.find_last_of( '\\' );
		buff = buff.substr( pos+1, buff.length()-pos-1 );
		sprintf( data.filename, "%s", buff.c_str() );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)data.filename, SysStringLen(bstrVal)+1, NULL, NULL );
	}

	Log( "OnSongChange! Item=%i, album=%s|artist=%s|duration=%s|filename=%s|status=%i|title=%s|", Item, data.album, data.artist, data.duration, data.filename, data.status, data.title );

	if ( SUCCEEDED(hr) )
	{
		data.msgID = KP_ONSONG;
		data.status = PLAYING;
 		DistributeData( &data );
	}
}

HWND kPilotInfo::GetClient()
{
	m_hPilot = FindWindow( WindowClass, WindowName );
	
	if (IsWindow(m_hPilot))
		return m_hPilot;
	else
		return NULL;
}

void kPilotInfo::DistributeData( kPilot2::sKPilot_base *pdata )
{
	COPYDATASTRUCT cpd;

	cpd.cbData = pdata->s_size;
	cpd.dwData = 0;
	cpd.lpData = pdata;

	if (GetClient())
		SendMessage( GetClient(), WM_COPYDATA, (WPARAM)m_hWnd, (LPARAM)&cpd );
}

HRESULT kPilotInfo::CMDPlay()
{
	CComPtr<IWMPControls> spControls;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) ) 
	{
		hr = m_pCore->get_controls(&spControls);
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls->play();
	}
	return hr;
}

HRESULT kPilotInfo::CMDPause()
{
	CComPtr<IWMPControls> spControls;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) ) 
	{
		hr = m_pCore->get_controls(&spControls);
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls->pause();
	}
	return hr;
}

HRESULT kPilotInfo::CMDStop()
{
	CComPtr<IWMPControls> spControls;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) ) 
	{
		hr = m_pCore->get_controls(&spControls);
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls->stop();
	}
	return hr;
}

HRESULT kPilotInfo::CMDNext()
{
	CComPtr<IWMPControls> spControls;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) ) 
	{
		hr = m_pCore->get_controls(&spControls);
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls->next();
	}
	return hr;
}

HRESULT kPilotInfo::CMDPrev()
{
	CComPtr<IWMPControls> spControls;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) ) 
	{
		hr = m_pCore->get_controls(&spControls);
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls->previous();
	}
	return hr;
}

HRESULT kPilotInfo::CMDVolumeUp()
{
	CComPtr<IWMPSettings> spSettings;
	long plVolume;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) )
	{
		hr = m_pCore->get_settings( &spSettings );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spSettings->get_volume( &plVolume );
	}
	if ( SUCCEEDED(hr) )
	{
		plVolume += 5;
		hr = spSettings->put_volume( plVolume );
	}
	return hr;
}

HRESULT kPilotInfo::CMDVolumeDown()
{
	CComPtr<IWMPSettings> spSettings;
	long plVolume;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) )
	{
		hr = m_pCore->get_settings( &spSettings );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spSettings->get_volume( &plVolume );
	}
	if ( SUCCEEDED(hr) )
	{
		plVolume -= 5;
		hr = spSettings->put_volume( plVolume );
	}
	return hr;
}

HRESULT kPilotInfo::CMDGetVolume( long *plVolume )
{
	CComPtr<IWMPSettings> spSettings;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) )
	{
		hr = m_pCore->get_settings( &spSettings );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spSettings->get_volume( plVolume );
	}
	return hr;
}

HRESULT kPilotInfo::CMDSetVolume( long lVolume )
{
	CComPtr<IWMPSettings> spSettings;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) )
	{
		hr = m_pCore->get_settings( &spSettings );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spSettings->put_volume( lVolume );
	}
	return hr;
}

KPILOTDATA kPilotInfo::TimeConvert( double time )
{
	int iPos, iSec, iMin, iHour;
	char *buff = (char*)malloc( 16 );
	KPILOTDATA data;

	iPos = (int)time;

	iSec = iPos % 60;		// Sekundy
	iMin = (iPos / 60)%60;		// Minuty
	iHour = iPos / 3600;	// Godziny

	if ( iHour == 0 )
	{
		if ( iMin == 0 )
		{
			if ( iSec < 10 )
				sprintf( buff, "0:0%i", iSec );
			else
				sprintf( buff, "0:%i", iSec );
		}
		else
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:0%i", iMin, iSec );
			else
				sprintf( buff, "%i:%i", iMin, iSec );
		}
	}
	else
	{
		if ( iMin == 0 )
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:00:0%i", iHour, iSec );
			else
				sprintf( buff, "%i:00:%i", iHour, iSec );
		}
		else if ( iMin < 10 )
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:0%i:0%i", iHour, iMin, iSec );
			else
				sprintf( buff, "%i:0%i:%i", iHour, iMin, iSec );
		}
		else
		{
			if ( iSec < 10 )
				sprintf( buff, "%i:%i:0%i", iHour, iMin, iSec );
			else
				sprintf( buff, "%i:%i:%i", iHour, iMin, iSec );
		}
	}

	memcpy( data.text, buff, strlen(buff)+1 );
	free(buff);

	return data;
}

HRESULT kPilotInfo::CMDTrackTime( kPilot2::sKPilot_Ex * pData )
{
	CComPtr<IWMPControls> spControls;
	CComPtr<IWMPControls3> spControls3;
	CComBSTR bstrTimecode;
	double dCurrentPos;
	kPilot2::sKPilot_Ex data;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) ) 
	{
		hr = m_pCore->get_controls(&spControls);
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls->QueryInterface(&spControls3);
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls3->get_currentPosition( &dCurrentPos );
	}
	if ( SUCCEEDED(hr) )
	{
		sprintf( data.szTime, "%s", TimeConvert( dCurrentPos ).text);
		data.time = dCurrentPos;
		*pData = *&data;
	}
	return hr;
}

HRESULT kPilotInfo::CMDTrackTimeRemain( kPilot2::sKPilot_Ex * pData )
{
	CComPtr<IWMPMedia> spMedia;
	CComPtr<IWMPControls> spControls;
	CComPtr<IWMPControls3> spControls3;
	double dDuration;
	double dTime;
	kPilot2::sKPilot_Ex data;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) )
	{	
		hr = m_pCore->get_controls( &spControls );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls->QueryInterface( &spControls3 );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls3->get_currentPosition( &dTime );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = m_pCore->get_currentMedia( &spMedia );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spMedia->get_duration( &dDuration );
	}
	if ( SUCCEEDED(hr) )
	{
		if (dTime)
            dDuration -= dTime;
		else
			dDuration = 0;
		sprintf( data.szTime, "%s", TimeConvert( dDuration ).text );
		data.time = dDuration;

		*pData = *&data;
	}
	return hr;
}

HRESULT kPilotInfo::CMDTrackTimePercent( kPilot2::sKPilot_Ex * pData )
{
	CComPtr<IWMPMedia> spMedia;
	CComPtr<IWMPControls> spControls;
	CComPtr<IWMPControls3> spControls3;
	double dDuration;
	double dTime;
	kPilot2::sKPilot_Ex data;
	int t;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) )
	{	
		hr = m_pCore->get_controls( &spControls );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls->QueryInterface( &spControls3 );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls3->get_currentPosition( &dTime );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = m_pCore->get_currentMedia( &spMedia );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spMedia->get_duration( &dDuration );
	}
	if ( SUCCEEDED(hr) )
	{
		if (dTime)
		{
			dDuration = (dTime/dDuration)*100;
            t = (int)dDuration;
		}
		else
		{
			t = 0;
			dDuration = 0;
		}
		
		sprintf( data.szTime, "%i", t );	
		data.time = dDuration;

		*pData = *&data;
	}
	return hr;
}

HRESULT kPilotInfo::CMDTrackTimePercentRemain( kPilot2::sKPilot_Ex * pData )
{
	CComPtr<IWMPMedia> spMedia;
	CComPtr<IWMPControls> spControls;
	CComPtr<IWMPControls3> spControls3;
	double dDuration;
	double dTime;
	kPilot2::sKPilot_Ex data;
	int t;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) )
	{	
		hr = m_pCore->get_controls( &spControls );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls->QueryInterface( &spControls3 );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spControls3->get_currentPosition( &dTime );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = m_pCore->get_currentMedia( &spMedia );
	}
	if ( SUCCEEDED(hr) )
	{
		hr = spMedia->get_duration( &dDuration );
	}
	if ( SUCCEEDED(hr) )
	{
		if (dTime)
		{
			dDuration = ((dDuration - dTime)/dDuration)*100;
			t = (int)dDuration;
		}
		else
		{
			t = 0;
			dDuration = 0;
		}
		sprintf( data.szTime, "%i", t );
		data.time = dDuration;

		*pData = *&data;
	}
	return hr;
}

HRESULT kPilotInfo::CMDPlayerVersion(  kPilot2::sKPilot_Ex * pData )
{
	BSTR bstrVersionInfo;
	HRESULT hr = S_OK;
	kPilot2::sKPilot_Ex data;
	char buff[64];

	if ( SUCCEEDED(hr) )
	{
		hr = m_pCore->get_versionInfo( &bstrVersionInfo );
	}
	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVersionInfo, -1, (LPSTR)data.szPlayer, SysStringLen(bstrVersionInfo)+1, NULL, NULL );
		sprintf( buff, "%s", data.szPlayer );
		sprintf( data.szPlayer, "Windows Media Player %s", buff );

		*pData = *&data;
	}
	return hr;
}

HRESULT kPilotInfo::CMDPlayerStatus( kPilot2::sKPilot_Ex * pData )
{
	BSTR bstrStatus;
	kPilot2::sKPilot_Ex data;
	HRESULT hr = S_OK;

	if ( SUCCEEDED(hr) )
	{
		hr = m_pCore->get_status( &bstrStatus );
	}
	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrStatus, -1, (LPSTR)data.szPlayer, SysStringLen(bstrStatus)+1, NULL, NULL );

		*pData = *&data;
	}

	return hr;
}

HRESULT kPilotInfo::CMDBitRate( kPilot2::sKPilot_Ex * pData )
{
	CComPtr<IWMPMedia> pMedia;
	HRESULT hr = S_OK;
	BSTR bstrVal;
	int i;
	kPilot2::sKPilot_Ex data;

	if (SUCCEEDED( hr ) )
	{
		hr = m_pCore->get_currentMedia( &pMedia );
	}

	if ( SUCCEEDED(hr) )
	{
		hr = pMedia->getItemInfo( L"Bitrate", &bstrVal );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)data.szBitrate, SysStringLen(bstrVal)+1, NULL, NULL );
		i = atoi(data.szBitrate);
		i /= 1000;
		sprintf( data.szBitrate, "%d", i );
		*pData = *&data;
	}

	return hr;
}

HRESULT kPilotInfo::CMDFileSize( kPilot2::sKPilot_Ex * pData )
{
	CComPtr<IWMPMedia> pMedia;
	HRESULT hr = S_OK;
	BSTR bstrVal;
	kPilot2::sKPilot_Ex data;

	if (SUCCEEDED( hr ) )
	{
		hr = m_pCore->get_currentMedia( &pMedia );
	}

	if ( SUCCEEDED(hr) )
	{
		hr = pMedia->getItemInfo( L"Filesize", &bstrVal );
	}

	if ( SUCCEEDED(hr) )
	{
		WideCharToMultiByte( CP_ACP, 0, bstrVal, -1, (LPSTR)data.szFileSize, SysStringLen(bstrVal)+1, NULL, NULL );
		
		sprintf( data.szFileSize, "%s", FileSizeConvert( data.szFileSize ).c_str() );
		*pData = *&data;
	}

	return hr;
}

void kPilotInfo::Log( const char *format, ... )
{
	char buff[1024];
	va_list ap;
	kPilot2::sKPilot_Log Log;

	va_start( ap, format );
	vsprintf( buff, format, ap );
	va_end( ap );

	sprintf( Log.Log, "%s", buff );
	Log.msgID = KP_LOG;

	DistributeData( &Log );
}

std::string kPilotInfo::FileSizeConvert( const char* size )
{
	int s, kilo, mega;
	char buff[16], t[5], v[5];
	t[0] = 0;
	v[0] = 0;

	if (!size)
		return 0;

	s = atoi( size );
	s /= 1024;	// kilobajty

	mega = s/1024;
	kilo = s % 1024;

	itoa( kilo, t, 10 );
	strncpy( v, t, 2 );
	v[2] = '\0';
	sprintf( buff, "%i,%s", mega, v);

	return std::string(buff);
}